; TOPVIEW.ASM
;
; Code fragments for detecting the presence of a multitasker and
; making .ASM programs "multitasker aware".
;

; This is part of the MASM 5.1/Turbo ASM HLL support. Change it
; support your actual language type and memory model.
;
.Model Tiny

INCLUDE	c:\comm\TOPVIEW.INC	; macros defined above

.Code

	Org	100h
START:
	jmp	MAIN

; This will hold the value of the base address of the display.
; It is set to the color text buffer physical address by default.
; You should determine if a mono system is in use and, if so,
; change the value of screen_segm to 0B000h prior to calling the
; detect_TopView procedure. detect_TopView will return a boolean
; False if a multitasker IS FOUND - this allows it's return value
; to be assigned directly to Borland's DirectVideo variable.
;
screen_address	label	dword
	screen_offs	dw	0
	screen_segm	dw	0B800h

; The values in these variables are moved into the AH register prior
; to calling int 16h to get keyboard input. detect_TopView will move
; the value of keyboard_read_fn into keyboard_check_fn if a multitasker
; is found. If you want to support the enhanced keyboard, you should
; determine if one exists and, if so, add 10h to each of these variables
; prior to calling detect_TopView.
;
keyboard_check_fn	db	01h
keyboard_read_fn	db	00h

; Determines the presence of a TopView compatible multitasker such as
; OMNIVIEW or DESQview and alters the variables above if one is found.
;
; Parms:	void
; Result:	Returns FALSE (0) if multitasker IS FOUND, else TRUE (1)
; Regs:		AX
;
detect_TopView	proc
		push	bx			; save bx
		push	di    			; save es:di
		push	es

		les	di, cs:screen_address	; get assumed physical address
		mov	ah, 0FEh 		; call TopView
		int	10h
		xor	ax, ax			; assume TopView found

		mov	bx, es
		cmp	bx, cs:screen_segm  	; see if ES was changed
		je	DTV_not_found     	; no

		mov	cs:screen_segm, bx	; yes: record virtual screen addr
		mov	cs:screen_offs, di
		mov	bl, cs:keyboard_read_fn ; give up time slice if no kbd input
		mov	cs:keyboard_check_fn, bl
		jmp	short DTV_done		; and return FALSE

	DTV_not_found:
		inc	ax			; else return TRUE

	DTV_done:
		pop	es			; restore all saved registers
		pop	di
		pop	bx
		ret				; and return
detect_TopView	endp


; Writes a string to virtual or physical video display memory. DOES NOT
; account for snowy CGAs. ASCIIZ string is default type.
;
; Parms:	AH = attribute
;		DH = row
;		DL = column
;		DS:SI = far address of string to write
; Result:	void
; Regs:		AX
;
write_string	proc
		push	bx
		push	cx
		push	dx
		push	si
		push	di
		push	es

		push	ax   			; compute offset of (X,Y)
		xor	ax, ax
		mov	es, ax
		mov	al, es:[44Ah]
		mul	dh
		xor	dh, dh
		add	ax, dx
		shl	ax, 1
		mov	cx, ax
		pop	ax

		@get_scr_addr			; get base address of video buffer
		add	di, cx			; add in desired offset

	;	lodsb				; for Pascal strings use this instead
	;	xor	cx, cx			; of the code below until WS_done
	;	mov	cl, al			; max Pascal string is 255 bytes
	;
	;WS_loop:
	;	movsb
	;	mov	al, ah
	;	stosb
	;	loop	WS_loop
	;
		xor	cx, cx
		dec	cx                      ; max ASCIIZ string is 64K

    WS_loop:
		lodsb	       			; get char to write
		or	al, al			; check for end of string
		jz	WS_done			; exit if end of ASCIIZ string found

		stosb				; else write char (need snow check)
		mov	al, ah			; and attribute (need snow check)
		stosb 				
		loop	WS_loop			; continue writing string

    WS_done:
		pop	es			; restore saved registers
		pop	di
		pop	si
		pop	dx
		pop	cx
		pop	bx
		ret				; and return
write_string	endp


; Gets keyboard input if available. Gives up time slice and waits if running
; under a multitasker.
;
; Parms:	void
; Result:	CF set if no input available (and no multitasker present)
; 			AX = extended scan code pair (AL = char)
; Regs:		AX
;
get_kbd_inp	proc
		mov	ah, cs:keyboard_check_fn
		test	ah, 1          	; B0 will always be 0 if "TopView" present
		jz	GKI_ReadIt

		int	16h		; else check for keyboard input
		jz	GKI_NoInput	; branch to set CF if none else
		mov	ah, cs:keyboard_read_fn

    GKI_ReadIt:
		int	16h		; read keyboard
		clc			; clear CF
		jmp	short GKI_done	; and return

    GKI_NoInput:
		stc

    GKI_done:
		ret
get_kbd_inp	endp


; test Main - comment out this code and remove reference to "START" in the
; final "end" if you want to use this code in another program. ALSO remove
; the Org 100h, etc. if you want to build an .EXE file instead of a .COM.
;
; This reads the first 100 chars from the keyboard (or until ESC is pressed)
; and prints them in 5 rows of red chars on the CRT.
;
string	db	2 dup(0)
MAIN	proc

		call	detect_topview		; set it all up

		lea	si, string		; get string address
		mov	cx, 100			; loop count
		xor	dx, dx			; and starting CRT (x,y) coordinates

	Main_loop:
		call	get_kbd_inp		; check kbd input
		jc	Main_loop		; loop if none

		cmp	ax, 011Bh		; check for ESC
		je   	Main_done		; exit if found

		mov	string, al		; put char in string
		mov	ah, 4			; use red on black attributes
		call	write_string		; write the char

		inc	dl			; next column
		cmp	dl, 20			; twenty yet?
		jne	Main_do			; no, just loop
		xor	dl, dl			; yes, wrap at 20
		inc	dh

	Main_do:
		loop	Main_loop		; print next char

	Main_done:
		mov	ax, 4C00h		; exit w/ ERRORLEVEL = 0
		int	21h
MAIN	endp


@CurSeg	ends
		end	start

